@@ -1,4 +1,5 @@
-# Copyright (c) 2004-2008 Infrae. All rights reserved.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2004-2012 Infrae. All rights reserved.
 # See also LICENSE.txt
 # $Id$
 
@@ -6,60 +7,16 @@ from datetime import datetime
 import time
 
 from zope.i18n.locales import locales
+from zope.i18n.interfaces import IUserPreferredLanguages
 
 _marker = object()
 
 
-def normalize_language(lang):
-    lang = lang.strip().lower()
-    lang = lang.replace('_', '-')
-    lang = lang.replace(' ', '')
-    return lang
-
-
 def get_locale_info(request):
-    assert request != _marker
-    accept_langs = request.get('HTTP_ACCEPT_LANGUAGE', '').split(',')
-
-    # Normalize lang strings
-    accept_langs = map(normalize_language, accept_langs)
-    # Then filter out empty ones
-    accept_langs = filter(None, accept_langs)
-
-    accepts = []
-
-    for index, lang in enumerate(accept_langs):
-        l = lang.split(';', 2)
-
-        quality = None
-
-        if len(l) == 2:
-            q = l[1]
-            if q.startswith('q='):
-                q = q.split('=', 2)[1]
-                quality = float(q)
-        else:
-            # If not supplied, quality defaults to 1
-            quality = 1.0
-
-        if quality == 1.0:
-            # ... but we use 1.9 - 0.001 * position to
-            # keep the ordering between all items with
-            # 1.0 quality, which may include items with no quality
-            # defined, and items with quality defined as 1.
-            quality = 1.9 - (0.001 * index)
-
-        accepts.append((quality, l[0]))
-
-    # Filter langs with q=0, which means
-    # unwanted lang according to the spec
-    # See: http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4
-    accepts = filter(lambda acc: acc[0], accepts)
-
-    accepts.sort()
-    accepts.reverse()
-
-    return accepts[0][1] if len(accepts) else 'en'
+    if request == _marker:
+        return 'en'
+    languages = IUserPreferredLanguages(request).getPreferredLanguages()
+    return (languages and languages[0]) or 'en'
 
 
 def get_locale_dates(request=_marker, locale=_marker):
